"""
Used to read data and return.
"""
import struct
from datetime import datetime
import time
import calendar

from unisense_xlink.sensors import Bme680Native, Bno055Native, SI1145Native, SI1145NativeRaw
import libscrc


def float_to_serial(float_data: float):
    return struct.pack('f', float_data)


def int_to_serial(int_data: int, signed: bool = False):
    if signed:
        fmt = 'i'
    else:
        fmt = 'I'
    return struct.pack(fmt, int_data)


def time_to_timestamp(dt: datetime):
    time_tuple = dt.utctimetuple()
    time_stamp = calendar.timegm(time_tuple)
    return time_stamp


def ts_to_serial(ts):
    return struct.pack('I', ts)


def time_to_serial(dt: datetime):
    ts = time_to_timestamp(dt)
    return ts_to_serial(ts)


def dsize(bytes_data):
    return struct.calcsize(bytes_data)


def bme_all(as_bytes: bool = False):
    sensor = Bme680Native()
    if as_bytes:
        bytes_data = struct.pack('fff', sensor()[0], sensor()[1], sensor()[2])
        return bytes_data
    return sensor()


def bme_temperature(as_bytes: bool = False):
    """:returns bme680 temperature"""
    data = bme_all()[0]
    if as_bytes:
        return float_to_serial(data)
    return data


def bme_pressure(as_bytes: bool = False):
    """:returns bme680 pressure"""
    data = bme_all()[1]
    if as_bytes:
        return float_to_serial(data)
    return data


def bme_humidity(as_bytes: bool = False):
    """:returns bme680 humidity"""
    data = bme_all()[2]
    if as_bytes:
        return float_to_serial(data)
    return data


def bno_all(as_bytes: bool = False):
    """bno all data has 22 float data which is 22*4 = 88 bytes long.
    so we do not have as_bytes option here."""
    sensor = Bno055Native()
    if as_bytes:
        return struct.pack('H', 255)
    return sensor()


def bno_temperature(as_bytes: bool = False):
    """:returns bno055 temperature"""
    data = bno_all()[0]
    if as_bytes:
        return float_to_serial(data)
    return data


def bno_acceleration(as_bytes: bool = False):
    """:returns bno055 acceleration"""
    data = bno_all()[1]
    if as_bytes:
        return struct.pack('fff', (data[0]), (data[1]), (data[2]))
    return data


def bno_magnetic(as_bytes: bool = False):
    """:returns bno055 magnetic"""
    data = bno_all()[2]
    if as_bytes:
        return struct.pack('fff', data[0], data[1], data[2])
    return data


def bno_gyro(as_bytes: bool = False):
    """:returns bno055 gyro"""
    data = bno_all()[3]
    if as_bytes:
        return struct.pack('fff', data[0], data[1], data[2])
    return data


def bno_euler(as_bytes: bool = False):
    """:returns bno055 euler"""
    data = bno_all()[4]
    if as_bytes:
        return struct.pack('fff', data[0], data[1], data[2])
    return data


def bno_quaternion(as_bytes: bool = False):
    """:returns bno055 quaternion"""
    data = bno_all()[5]
    if as_bytes:
        return struct.pack('fff', data[0], data[1], data[2])
    return data


def bno_linear_acceleration(as_bytes: bool = False):
    """:returns bno055 linear_acceleration"""
    data = bno_all()[6]
    if as_bytes:
        return struct.pack('fff', data[0], data[1], data[2])
    return data


def bno_gravity(as_bytes: bool = False):
    """:returns bno055 gravity"""
    data = bno_all()[7]
    if as_bytes:
        return struct.pack('fff', data[0], data[1], data[2])
    return data


def si_all(with_exceptions: bool = False, as_bytes: bool = False):
    """:returns all si data vis:ir:uv"""
    if with_exceptions:
        sensor = SI1145NativeRaw()
    else:
        sensor = SI1145Native()
    if as_bytes:
        bytes_data = struct.pack('IIH', sensor()[0], sensor()[1], sensor()[2])
        return bytes_data
    return sensor()


def si_vis(with_exceptions: bool = False, as_bytes: bool = False):
    """si1145:vis"""
    if with_exceptions:
        data = si_all(with_exceptions=True)[0]
    else:
        data = si_all()[0]
    if as_bytes:
        return struct.pack('I', data)
    return data


def si_ir(with_exceptions: bool = False, as_bytes: bool = False):
    """si1145:ir"""
    if with_exceptions:
        data = si_all(with_exceptions=True)[1]
    else:
        data = si_all()[1]
    if as_bytes:
        return struct.pack('I', data)
    return data


def si_uv(with_exceptions: bool = False, as_bytes: bool = False):
    """si1145:uv"""
    if with_exceptions:
        data = si_all(with_exceptions=True)[2]
    else:
        data = si_all()[2]
    if as_bytes:
        return struct.pack('H', data)
    return data


def repr_hex(bytes_obj: bytes):
    return bytes_obj.hex()


def calc_crc(data, return_type: str = 'hex'):
    """ returns little endian modbus crc16
    :param data bytes data
    :param return_type str hex,bytes
    :return crc16 modbus
    """
    res = struct.pack('h', libscrc.modbus(data))
    if return_type == 'bytes':
        return res
    return res.hex()



r"""
                     _            _____
     /\             | |     /\   |_   _|
    /  \    ______ _| |_   /  \    | |
   / /\ \  |_  / _` | __| / /\ \   | |
  / ____ \  / / (_| | |_ / ____ \ _| |_
 /_/    \_\/___\__,_|\__/_/    \_\_____|
"""

__title__ = "UniSense Xlink"
__version__ = "0.1.1"
__author__ = "Yaakov Azat"
__license__ = "AzatAI License"
__copyright__ = "Copyright 2020-2021 Azat Artificial Intelligence, LLP"

# Version synonym
VERSION = __version__

# Header encoding (see RFC5987)
HTTP_HEADER_ENCODING = "iso-8859-1"

# Default datetime input and output formats
ISO_8601 = "iso-8601"


# python core imports
from typing import Tuple
from datetime import datetime
# pydantic imports
from pydantic import BaseModel

from unisense_xlink import core
from unisense_xlink.core import time_to_serial

"""
Data Length

temperature: int 4
float : 4
"""


class BmeDataBase(BaseModel):
    temperature: float = core.bme_temperature()
    pressure: float = core.bme_pressure()
    humidity: float = core.bme_humidity()


class BmeDataBaseTime(BmeDataBase):
    updated: datetime = datetime.utcnow()


class BmeDataBin(BaseModel):
    temperature: bytes = core.bme_temperature(as_bytes=True)
    pressure: bytes = core.bme_pressure(as_bytes=True)
    humidity: bytes = core.bme_humidity(as_bytes=True)


class BmeDataAll(BaseModel):
    data: bytes = core.bme_all(as_bytes=True)


class BmeDataBinTime(BmeDataBin):
    updated: bytes = time_to_serial(datetime.utcnow())


class BnoDataBase(BaseModel):
    temperature: float = core.bno_temperature()
    acceleration: Tuple[float, float, float] = core.bno_acceleration()
    magnetic: Tuple[float, float, float] = core.bno_magnetic()
    gyro: Tuple[float, float, float] = core.bno_gyro()
    euler: Tuple[float, float, float] = core.bno_euler()
    quaternion: Tuple[float, float, float] = core.bno_quaternion()
    linear_acceleration: Tuple[float, float, float] = core.bno_linear_acceleration()
    gravity: Tuple[float, float, float] = core.bno_gravity()


class BnoDataBaseTime(BnoDataBase):
    updated: datetime = datetime.utcnow()


class BnoDataBin(BaseModel):
    temperature: bytes = core.bno_temperature(as_bytes=True)
    acceleration: bytes = core.bno_acceleration(as_bytes=True)
    magnetic: bytes = core.bno_magnetic(as_bytes=True)
    gyro: bytes = core.bno_gyro(as_bytes=True)
    euler: bytes = core.bno_euler(as_bytes=True)
    quaternion: bytes = core.bno_quaternion(as_bytes=True)
    linear_acceleration: bytes = core.bno_linear_acceleration(as_bytes=True)
    gravity: bytes = core.bno_gravity(as_bytes=True)


class BnoDataBinTime(BnoDataBin):
    updated: bytes = time_to_serial(datetime.utcnow())


class SiDataBase(BaseModel):
    vis: int = core.si_vis()
    ir: int = core.si_ir()
    uv: int = core.si_uv()


class SiDataBaseException(BaseModel):
    vis: int = core.si_vis(with_exceptions=True)
    ir: int = core.si_ir(with_exceptions=True)
    uv: int = core.si_uv(with_exceptions=True)


class SiDataBaseTime(SiDataBase):
    updated: datetime = datetime.utcnow()


class SiDataBaseExceptionTime(SiDataBaseException):
    updated: datetime = datetime.utcnow()


class SiDataBin(BaseModel):
    vis: bytes = core.si_vis(with_exceptions=False, as_bytes=True)
    ir: bytes = core.si_ir(with_exceptions=False, as_bytes=True)
    uv: bytes = core.si_uv(with_exceptions=False, as_bytes=True)


class SiDataAll(BaseModel):
    data: bytes = core.si_all(as_bytes=True)


class SiDataBinTime(SiDataBin):
    updated: bytes = time_to_serial(datetime.utcnow())



# bme680 import
import bme680

# bno055 imports
import adafruit_bno055
import board
# SI1145 imports
import SI1145.SI1145 as SI1145

i2c = board.I2C()


class Bme680Native:
    """'temperature,pressure,humidity'"""

    def __init__(self) -> None:
        try:
            self.sensor = bme680.BME680(bme680.I2C_ADDR_PRIMARY)
        except (RuntimeError, IOError):
            self.sensor = bme680.BME680(bme680.I2C_ADDR_SECONDARY)

        self.sensor.set_humidity_oversample(bme680.OS_2X)
        self.sensor.set_pressure_oversample(bme680.OS_4X)
        self.sensor.set_temperature_oversample(bme680.OS_8X)
        self.sensor.set_filter(bme680.FILTER_SIZE_3)

        self.temperature = self.sensor.data.temperature
        self.pressure = self.sensor.data.pressure
        self.humidity = self.sensor.data.humidity

    def __call__(self, *args, **kwargs):
        return self.temperature, self.pressure, self.humidity


class Bno055Native:
    """Returns data;
    self.temperature,  float
    self.acceleration,   (float,float,float)
    self.magnetic,   (float,float,float)
    self.gyro,    (float,float,float)
    self.euler,   (float,float,float)
    self.quaternion,    (float,float,float)
    self.linear_acceleration,    (float,float,float)
    self.gravity    (float,float,float)
    """

    def __init__(self):
        self.sensor = adafruit_bno055.BNO055_I2C(i2c)
        self.temperature = self.sensor.temperature
        self.acceleration = self.sensor.acceleration
        self.magnetic = self.sensor.magnetic
        self.gyro = self.sensor.gyro
        self.euler = self.sensor.euler
        self.quaternion = self.sensor.quaternion
        self.linear_acceleration = self.sensor.linear_acceleration
        self.gravity = self.sensor.gravity

    def __call__(self, *args, **kwargs):
        return (self.temperature,
                self.acceleration,
                self.magnetic,
                self.gyro,
                self.euler,
                self.quaternion,
                self.linear_acceleration,
                self.gravity
                )


class SI1145Native:
    """vis,ir,uv : int, int ,short [I,I,H]"""

    def __init__(self):
        self.sensor = SI1145.SI1145()
        try:
            self.vis = self.sensor.readVisible()
        except:
            self.vis = 0

        try:
            self.ir = self.sensor.readIR()
        except:
            self.ir = 0

        try:
            self.uv = self.sensor.readUV()
        except:
            self.uv = 0

    def __call__(self, *args, **kwargs):
        return (self.vis,
                self.ir,
                self.uv
                )


class SI1145NativeRaw:
    """vis,ir,uv : int, int ,short [I,I,H]"""

    def __init__(self):
        self.sensor = SI1145.SI1145()
        self.vis = self.sensor.readVisible()
        self.ir = self.sensor.readIR()
        self.uv = self.sensor.readUV()

    def __call__(self, *args, **kwargs):
        return (self.vis,
                self.ir,
                self.uv
                )



#!/usr/bin/env python3

import os
import re
import shutil
import sys

from setuptools import find_packages, setup


def read(f):
    return open(f, "r", encoding="utf-8").read()


def get_version(package):
    """
    Retuen package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, "__init__.py")).read()
    return re.search("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


version = get_version("unisense_xlink")

if sys.argv[-1] == "publish":
    if os.system("pip freeze | grep twine"):
        print("Twine is not installed.\nUse `pip install twine`.\nExiting.")
        sys.exit()
    if os.system("pip list | grep wheel"):
        print("Wheel is not installed.\nUse `pip install wheel`.\nExiting.")
        sys.exit()
    os.system("python setup.py sdist bdist_wheel")
    print("You probably want to also tag the version now:")
    print("  git tag -a v%s -m 'version %s'" % (version, version))
    print("  git push --tags")
    shutil.rmtree("dist")
    shutil.rmtree("build")
    shutil.rmtree("unisense_xlink.egg-info")
    sys.exit()

if sys.argv[-1] == "git":
    cmd = "git tag -a v%s -m 'version %s'" % (version, version)
    os.system(cmd)
    os.system("git push --tags")
    x = input("Push new version to Github? (y/n)")
    if x == "y":
        os.system("git add .")
        cmd = "git commit -m 'version %s'" % (version)
        os.system(cmd)
        b = input("which branch to push? (default: master)")
        g_cmd = f"git branch | grep {b}"
        g = os.system(g_cmd)
        if b == "":
            os.system("git push origin master")
        elif g is not None:
            cmd = f"git push origin {b}"
            os.system(cmd)
    sys.exit()

# 'setup.py publish' shortcut.
if sys.argv[-1] == 'upload':
    os.system('python setup.py sdist')
    os.system('twine upload dist/*')
    os.system('rm -rf dist')
    os.system('rm -rf *.egg-info')
    sys.exit()

setup(
    name="unisense_xlink",
    version=version,
    url="https://github.com/AzatAI/unisense_xlink",
    license="AzatAI Licence",
    description="Connect and setup AzatAI django projects, made easy.",
    long_description=read("README.md"),
    author="AzatAI",
    author_email="info@azat.ai",
    packages=find_packages(exclude=["tests*"]),
    include_package_data=True,
    install_requires=[
        'Adafruit-Blinka',
        'adafruit-circuitpython-bno055',
        'adafruit-circuitpython-busdevice',
        'adafruit-circuitpython-register',
        'Adafruit-GPIO',
        'Adafruit-PlatformDetect',
        'Adafruit-PureIO',
        'bme680',
        'pyusb',
        'PyYAML',
        'regex',
        'rpi-ws281x',
        'RPi.GPIO',
        'shellingham',
        'smbus',
        'smbus2',
        'typing-extensions',
        'Werkzeug',
        'zipp',
        'SI1145'
    ],
    python_requires=">=3.6",
    zip_safe=False,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3 :: Only",
        "Topic :: Internet :: WWW/HTTP",
    ],
    project_urls={
        "Official": "https://azat.ai",
        "Source": "https://github.com/AzatAI/unisense_xlink",
    },
)
